# 1.1.2 数据类型与变量

## Python内置数据类型

如果让您解释一下什么是计算机的话，您可能会说计算机顾名思义就是可以做数学计算的机器。的确如此，因此，计算机程序理所当然地可以处理各种数值。但是，计算机能处理的远不止数值，还可以处理文本、图形、视频等各种各样的数据，不同的数据，需要定义不同的数据类型。在 Python 中，能够直接处理的数据类型称为内置数据类型，内置数据类型有：**整数、浮点数、字符串和布尔值**。

### 整数

`Python`可以处理任意大小的整数，当然包括负整数，在程序中的表示方法和数学上的写法一模一样，例如：`1，100，-8080，0`等等。

由于计算机使用二进制，所以，有时候用十六进制表示整数比较方便，十六进制用`0x`前缀和`0-9，a-f`表示，例如：`0xff00，0xa5b4c3d2`等等。


### 浮点数

浮点数也就是小数，之所以称为浮点数，是因为按照科学记数法表示时，一个浮点数的小数点位置是可变的，比如，`1.23x10^9`和`12.3x10^8`是完全相等的。浮点数可以用数学写法，如`1.23，3.14，-9.01`等等。但是对于很大或很小的浮点数，就必须用科学计数法表示，把`10`用`e`替代，`1.23x10^9`就是`1.23e9`，或者`12.3e8，0.000012`可以写成`1.2e-5`等等。



### 字符串

字符串是以单引号`'`或双引号`"`括起来的任意文本，比如`'abc'，"xyz"`等等。请注意，`''`或`""`本身只是一种表示方式，不是字符串的一部分，因此，字符串`'abc'`只有`a`，`b`，`c`这3个字符。如果`'`本身也是一个字符，那就可以用`""`括起来，比如`"I'm OK"`包含的字符是`I`，`'`，`m`，`空格`，`O`，`K`这6个字符。

如果字符串内部既包含`'`又包含`"`怎么办？可以用转义字符`\ `来标识，比如：`'I\'m \"OK\"!'`

表示的字符串内容是：
`I'm "OK"!`

转义字符`\`可以转义很多字符，比如`\n `表示换行，`\t `表示制表符，字符`\ `本身也要转义，所以`\\`表示的字符就是`\ `，可以在 Python 的交互式命令行用`print()`打印字符串看看：

```
>>> print('I\'m ok.')
I'm ok.
>>> print('I\'m learning\nPython.')
I'm learning
Python.
>>> print('\\\n\\')
\
\
>>> print('\tHello')
    Hello
```

如果字符串里面有很多字符都需要转义，就需要加很多`\ `，为了简化， Python 还允许用`r''`表示`''`内部的字符串默认不转义，可以自己试试：

```
>>> print('\\\t\\')
\       \
>>> print(r'\\\t\\')
\\\t\\
```

部分转义字符：

|转义字符   | 描述  |
| :-:  | :-:  |
| \  | (在行尾时)	续行符  |
| \\\  | 反斜杠符号  |
| \'  | 单引号  |
| \"  | 双引号  |
| \a  | 响铃  |
| \b  | 退格(Backspace)  |
|\n|	换行|
|\t|	横向制表符|


对于中文字符串的表示，为保证  Python 程序处理中文字符时不乱码，需要在 Python 程序最前面加上`#coding = utf-8`,例如：

```python
>>> #coding=utf-8
>>> a = "你好世界！"
>>> print(a)
你好世界！
```

######字符串运算符

字符串可以使用`+`号和`*`号进行运算。可以组合和复制，运算后会生成一个新的字符串。

|Python 表达式|	结果|	描述|
| :-:  | :-:  | :-:  |
|len("abcd")|	4|	计算元素个数|
|"123" + "456"|	"123456"|	连接|
|"Hi!" * 4|	'Hi!Hi!Hi!Hi!'|	复制|
|"3" in "123"|	True|	元素是否存在|

######字符串内置函数

字符串包含了以下内置函数

|  函数名 | 描述  |
| :-:  | :-:  |
| len(list)|计算字符串长度|
|	string.capitalize()|字母字符串首字母大写|
| 	string.split(str)|将字符串按`str`的形式分割|
| 	str(a)|将变量转换为字符串|

```python
#使用逗号运算符给多个变量赋值
t1,t2 = "1-2-3-4","abcd"
print(len(t1))
print(t2.capitalize())
print(t1.split("-"))
#创建一个整型变量a
a = 123
print(str(a))
```
以上实例输出结果：
```pyhton
7
Abcd
['1','2','3','4']
"123"
```

### 布尔值

布尔值和布尔代数的表示完全一致，一个布尔值只有`True、False`两种值，要么是`True`，要么是`False`，在Python中，可以直接用`True`、`False`表示布尔值（请注意大小写），也可以通过布尔运算计算出来：

```python
>>> True
True
>>> False
False
>>> 3 > 2
True
>>> 3 > 5
False
```

布尔值可以用`and`、`or`和`not`运算。

`and`运算是**与运算**，只有所有都为`True`，`and`运算结果才是`True`：

```
>>> 5 > 3 and 3 > 1
True
```

`or`运算是**或运算**，只要其中有一个为`True`，`or`运算结果就是`True`：

```
>>> 5 > 3 or 1 > 3
True
```

`not`运算是**非运算**，它是一个单目运算符，把`True`变成`False`，`False`变成`True`：

```
>>> not 1 > 2
True
```

