# 5.2.6 好还不够，我要最好---支持向量机

## 线性可分支持向量机

例如逻辑回归这种广义线性模型对数据进行分类时，其实本质就是找到一条 **决策边界** ，然后将数据分成两个类别(边界的一边是一种类别，另一边是另一种类别)。如下图中的两条线是两种模型产生的 **决策边界** ：

![](20.jpg)

图中的绿线与黄线都能很好的将图中的红点与蓝点给区分开。但是，哪条线的泛化性更好呢？可能你不太了解泛化性，也就是说，这条直线，不仅需要在训练集 **(已知的数据)** 上能够很好的将红点跟蓝点区分开来，还要在测试集 **(未知的数据)** 上将红点和蓝点给区分开来。


假如经过训练，我们得到了黄色的这条决策边界用来区分数据，这个时候又来了一个数据，即黑色的点，那么你觉得黑色的点是属于红的这一类，还是蓝色的这一类呢？

![](21.jpg)

如上图，根据黄线的划分标准，黑色的点应该属于红色这一类。可是，肉眼很容易发现，黑点离蓝色的点更近，它应该是属于蓝色的点。这就说明，黄色的这条直线它的泛化性并不好，它对于未知的数据并不能很好的进行分类。那么，如何得到一条泛化性好的直线呢？这个就是支持向量机考虑的问题。

### 基本思想

支持向量机的思想认为，一条决策边界它如果要有很好的泛化性，它需要满足一下以下两个条件：

- 能够很好的将样本划分
- 离最近的样本点最远

比如下图中的黑线：

![](22.jpg)

它能够正确的将红点跟蓝点区分开来，而且，它还保证了对未知样本的容错率，因为它离最近的红点跟蓝点都很远，这个时候，再来一个数据，就不会出现之前黄色决策边界的错误了。

![](23.jpg)

无论新的数据出现在哪个位置，黑色的决策边界都能够很好的给它进行分类，这个就是支持向量机的基本思想。

### 间隔与支持向量

在样本空间中，决策边界可以通过如下线性方程来描述：

<center>
$$
w^Tx+b=0
$$
</center>
<br>

其中$$w=(w_1,w_2,..,w_d)$$为法向量，决定了决策边界的方向。$$b$$为位移项，决定了决策边界与原点之间的距离。显然，决策边界可被法向量和位移确定，可以将其表示为$$(w,b)$$。样本空间中的任意一个点$$x$$，到决策边界$$(w,b)$$的距离可写为：

<center>
$$
r=\frac{|w^Tx+b|}{||w||}
$$
</center>
<br>

假设决策边界$$(w,b)$$能够将训练样本正确分类，即对于任何一个样本点$$(x_i,y_i)$$，若它为正类，即$$y_i=+1$$时，$$w^Tx+b\geq +1$$。若它为负类，即$$y_i=-1$$时，$$w^Tx+b\leq -1$$。

![](24.jpg)

如图中，距离最近的几个点使两个不等式的等号成立，它们就被称为支持向量，即图中两条黄色的线。两个异类支持向量到超平面的距离之和为：

<center>
$$
r=\frac{2}{||w||}
$$
</center>
<br>

它被称为间隔，即蓝线的长度。欲找到具有“最大间隔”的决策边界，即黑色的线，也就是要找到能够同时满足如下式子的$$w$$与$$b$$:

![](25.jpg)

显然，为了最大化间隔，仅需最大化$$||w||^{-1}$$，这等价于最小化$$||w||^2$$，于是，条件可以重写为：

<center>
$$
min \frac{1}{2}||w||^2
$$
</center>

<center>
$$
s.t. y_i(w^Tx+b)\geq1
$$
</center>

这就是线性可分支持向量机的基本型。

### 对偶问题

现在已经知道了支持向量机的基本型，问题本身是一个凸二次规划问题，可以用现成的优化计算包求解，不过可以用更高效的方法。支持向量机的模型如下：

<center>
$$
min\frac{1}{2}||W||^2
$$
</center>

<center>
$$
s.t. y_i(w^Tx+b)\geq1
$$
</center>
<br>

对两式子使用拉格朗日乘子法可得到其对偶问题。具体为，对式子的每条约束添加拉格朗日乘子$$\alpha\geq0$$，则该问题的拉格朗日函数可写为：

<center>
$$
L(w,b,\alpha)=\frac{1}{2}||w||^2+\sum^m_{i=1}\alpha_i(1-y_i(w^Tx_i+b))......(1)
$$
</center>
<br>

其中$$\alpha=(\alpha_1,\alpha_2,..,\alpha_m)$$。令$$L(w,b,\alpha)$$对$$w$$和$$b$$的偏导为零可得：

<center>
$$
w=\sum_{i=1}^m\alpha_iy_ix_i......(2)
$$
</center>

<center>
$$
0=\sum_{i=1}^m\alpha_iy_i......(3)
$$
</center>
<br>

将式子$$2$$带入式子$$1$$，则可将$$w$$和$$b$$消去，再考虑式子$$3$$的约束，就可得到原问题的对偶问题：

<center>
$$
max(\sum^m_{i=1}\alpha_i-\frac{1}{2}\sum_{i=1}^m\sum_{j=1}^m\alpha_i\alpha_jy_iy_jx_i^Tx_j)
$$
</center>

<center>
$$
s.t. \sum_{i=1}^m\alpha_iy_i=0
$$
</center>

<center>
$$
\alpha_i\geq0
$$
</center>
<br>

解出$$\alpha$$后，求出$$w$$与$$b$$即可得到模型：

<center>
$$
f(x)=w^Tx+b=\sum_{i=1}^m\alpha_iy_ix_i^Tx+b......(4)
$$
</center>
<br>

从对偶问题解出的$$\alpha_i$$是拉格朗日乘子，它恰对应着训练样本$$(x_i,y_i)$$，由于原问题有不等式的约束，因此上述过程需满足KKT(Karush-Kuhn-Tucker)条件，即要求：

<center>
$$
\begin{cases}
\alpha_i\geq0\\
y_if(x_i)-1\geq0\\
\alpha_i(y_if(x_i)-1)=0
\end{cases}
$$
</center>

于是，对任意训练样本$$(x_i,y_i)$$，总有$$\alpha_i=0$$或$$y_if(x_i)=1$$。若$$\alpha_i=0$$，则该样本将不会出现在式子$$4$$中，也就不会对$$f(x)$$，有任何影响。若$$\alpha_i>0$$，则必有$$y_if(x_i)=1$$，所对应的样本点位于最大间隔边界上，是一个支持向量。这显示出支持向量机的一个重要性质: 训练完后，大部分的训练样本都不需要保留，最终模型仅与支持向量有关。

## 线性支持向量机

假如现在有一份数据分布如下图：

![](26.jpg)

按照线性可分支持向量机的思想，黄色的线就是最佳的决策边界。很明显，这条线的泛化性不是很好，造成这样结果的原因就是数据中存在着异常点，那么如何解决这个问题呢，支持向量机引入了软间隔最大化的方法来解决。

所谓的软间隔，是相对于硬间隔说的，刚刚在间隔与支持向量中提到的间隔就是硬间隔。

接着我们再看如何可以软间隔最大化呢？SVM 对训练集里面的每个样本$$(x_i,y_i)$$引入了一个松弛变量$$\xi_i\geq0$$，使函数间隔加上松弛变量大于等于$$1$$，也就是说：

<center>
$$
y_i(w^Tx+b)\geq1-\xi
$$
</center>
<br>

对比硬间隔最大化，可以看到我们对样本到超平面的函数距离的要求放松了，之前是一定要大于等于$$1$$，现在只需要加上一个大于等于$$0$$的松弛变量能大于等于$$1$$就可以了。也就是允许支持向量机在一些样本上出错，如下图：

![](27.jpg)

当然，松弛变量不能白加，这是有成本的，每一个松弛变量$$\xi_i$$, 对应了一个代价$$\xi_i$$，这个就得到了我们的软间隔最大化的支持向量机，模型如下：

<center>
$$
min \frac{1}{2}||w||^2+C\sum^{m}_{i=1}\xi_i
$$
</center>

<center>
$$
s.t. y_i(w^Tx_i+b)\geq1-\xi_i
$$
</center>

<center>
$$
\xi_i\geq0
$$
</center>
<br>

这里，$$C>0$$为惩罚参数，可以理解为一般回归和分类问题正则化时候的参数。$$C$$越大，对误分类的惩罚越大，$$C$$越小，对误分类的惩罚越小。也就是说，希望权值的二范数尽量小，误分类的点尽可能的少。$$C$$是协调两者关系的正则化惩罚系数。在实际应用中，需要调参来选择。

同样的，使用拉格朗日函数将软间隔最大化的约束问题转化为无约束问题，利用相同的方法得到数学模型：

<center>
$$
max(\sum^m_{i=1}\alpha_i-\frac{1}{2}\sum_{i=1}^m\sum_{j=1}^m\alpha_i\alpha_jy_iy_jx_i^Tx_j)
$$
</center>

<center>
$$
s.t. \sum_{i=1}^m\alpha_iy_i=0
$$
</center>

<center>
$$
0\leq\alpha_i\leq C
$$
</center>
<br>

这就是软间隔最大化时的线性支持向量机的优化目标形式，和之前的硬间隔最大化的线性可分支持向量机相比，我们仅仅是多了一个约束条件:

<center>
$$
0\leq\alpha_i\leq C
$$
</center>

## sklearn中的支持向量机

`SVR`是`sklearn`中提供的支持向量机回归器，`SVR`的构造函数中有四个常用的参数可以设置：

	 kernel：采用核函数类型，默认为`rbf`。可选参数有：`linear`（线性核函数）,`poly`（多项式核函数）,`rbf`：（径像核函数/高斯核）。
	 C：错误项的惩罚参数C，默认为1.0。
	 gamma:核函数系数，只对`rbf`,`poly`有效。如果gamma为auto，代表其值为样本特征数的倒数，即1/n_features。
	 epsilon:误差精度。


`SVR`类中的`fit`函数用于训练模型，`fit`函数有两个向量输入：

- `X`：大小为**[样本数量,特征数量]**的`ndarray`，存放训练样本
- `Y`：值为整型，大小为**[样本数量]**的`ndarray`，存放训练样本的标签值

`SVR`类中的`predict`函数用于预测，返回预测值，`predict`函数有一个向量输入：

- `X`：大小为**[样本数量,特征数量]**的`ndarray`，存放预测样本

`SVR`的使用代码如下：

```python
from sklearn.svm import SVR
svr = SVR()
svr.fit(X_train, Y_train)
predict = lr.predict(X_test)
```

`SVC`是`sklearn`中提供的支持向量机分类器，用法和`SVR`一致，`SVC`的使用代码如下:

```python
from sklearn.svm import SVC
svr = SVC()
svr.fit(X_train, Y_train)
predict = lr.predict(X_test)
```


如果想动手实现支持向量机算法，并掌握如何使用 sklearn 来解决实际问题，可以尝试进入链接进行实战：https://www.educoder.net/shixuns/b6yi97f2/challenges
