# 5.2.8 知错能改善莫大焉---Adaboost

## Boosting

**提升方法**基于这样一种思想：对于一个复杂任务来说，将多个专家的判断进行适当的综合所得出的判断，要比其中任何一个专家单独的判断好。

历史上，`Kearns`和`Valiant`首先提出了**强可学习**和**弱可学习**的概念。指出：在`PAC`学习的框架中，一个概念，如果存在一个多项式的学习算法能够学习它，并且正确率很高，那么就称这个概念是强可学习的；一个概念，如果存在一个多项式的学习算法能够学习它，学习的正确率仅比随机猜测略好，那么就称这个概念是弱可学习的。非常有趣的是`Schapire`后来证明强可学习与弱可学习是等价的，也就是说，在`PAC`学习的框架下，一个概念是强可学习的充分必要条件是这个概念是弱可学习的。

这样一来，问题便成为，在学习中，如果已经发现了**弱学习算法**，那么能否将它**提升**为**强学习算法**。大家知道，发现弱学习算法通常要比发现强学习算法容易得多。那么如何具体实施提升，便成为开发提升方法时所要解决的问题。

与`bagging`不同，`boosting`采用的是一个串行训练的方法。首先，它训练出一个**弱分类器**，然后在此基础上，再训练出一个稍好点的**弱分类器**，以此类推，不断的训练出多个弱分类器，最终再将这些分类器相结合，这就是`boosting`的基本思想，流程如下图：

![](30.jpg)

可以看出，子模型之间存在强依赖关系，必须串行生成。`boosting`是利用不同模型的相加，构成一个更好的模型，求取模型一般都采用序列化方法，后面的模型依据前面的模型。

## Adaboost原理

对提升方法来说，有两个问题需要回答：**一是在每一轮如何改变训练数据的权值或概率分布；二是如何将弱分类器组合成一个强分类器。**关于第`1`个问题，AdaBoost的做法是，**提高那些被前一轮弱分类器错误分类样本的权值，而降低那些被正确分类样本的权值**。这样一来，那些没有得到正确分类的数据，由于其权值的加大而受到后一轮的弱分类器的更大关注。于是，分类问题被一系列的弱分类器“分而治之”。至于第`2`个问题，即弱分类器的组合，AdaBoost采取**加权多数表决的方法，加大分类误差率小的弱分类器的权值，使其在表决中起较大的作用，减小分类误差率大的弱分类器的权值，使其在表决中起较小的作用**。

## Adaboost算法流程

`AdaBoost`是`AdaptiveBoost`的缩写，表明该算法是具有适应性的提升算法。

算法的步骤如下：

`1`. 给每个训练样本$$(x_1,x_2,..,x_N)$$分配权重，初始权重$$w_1$$均为$$1/N$$

`2`. 针对带有权值的样本进行训练，得到模型$$G_m$$（初始模型为$$G
_1$$）

`3`. 计算模型$$G_m$$的误分率：

$$
e_m=\sum_i^Nw_iI(y_i\neq G_M(X_i))
$$

其中：

$$
I(y_i\neq G_M(X_i)
$$

为指示函数，表示括号内成立时函数值为`1`，否则为`0`

`4`. 计算模型$$G_m$$的系数：

$$
\alpha_m=\frac{1}{2}\log[\frac{1-e_m}{e_m}]
$$

`5`. 根据误分率$$e$$和当前权重向量$$w_m$$更新权重向量：

$$
w_{m+1,i}=\frac{w_m}{z_m}exp(-\alpha_my_iG_m(x_i))
$$


其中$$Z_m$$为规范化因子：

$$
z_m=\sum_{i=1}^mw_{mi}exp(-\alpha_my_iG_m(x_i))
$$


`6`. 计算组合模型$$f(x)=\sum_{m=1}^M\alpha_mG_m(x_i)$$的误分率

`7`. 当组合模型的误分率或迭代次数低于一定阈值，停止迭代；否则，回到步骤 `2`

## sklearn中的Adaboost

`AdaBoostClassifier`是`skleran`中Adaboost分类器的实现，`AdaBoostClassifier`的构造函数中有四个常用的参数可以设置：

- `algorithm`：这个参数只有 AdaBoostClassifier 有。主要原因是scikit-learn 实现了两种 Adaboost 分类算法，`SAMME`和`SAMME.R`。两者的主要区别是弱学习器权重的度量，`SAMME.R`使用了概率度量的连续值，迭代一般比`SAMME`快，因此 AdaBoostClassifier 的默认算法`algorithm`的值也是`SAMME.R`。
- `n_estimators`：弱学习器的最大迭代次数。一般来说`n_estimators`太小，容易欠拟合，`n_estimators`太大，又容易过拟合，一般选择一个适中的数值。默认是`50`。
- `learning_rate`：AdaBoostClassifier 和 AdaBoostRegressor 都有，即每个弱学习器的权重缩减系数`ν`，默认为`1.0`。
- `base_estimator`：弱分类学习器或者弱回归学习器。理论上可以选择任何一个分类或者回归学习器，不过需要支持样本权重。我们常用的一般是 CART 决策树或者神经网络 MLP。


和`sklearn`中其他分类器一样，AdaBoostClassifier 类中的`fit`函数用于训练模型，`fit`函数有两个向量输入：

- `X`：大小为**[样本数量,特征数量]**的`ndarray`，存放训练样本
- `Y`：值为整型，大小为**[样本数量]**的`ndarray`，存放训练样本的分类标签

AdaBoostClassifier 类中的`predict`函数用于预测，返回预测标签，`predict`函数有一个向量输入：

`X`：大小为**[样本数量,特征数量]**的`ndarray`，存放预测样本
AdaBoostClassifier的使用代码如下：

```python
ada=AdaBoostClassifier(n_estimators=5,learning_rate=1.0)
ada.fit(train_data,train_label)
predict = ada.predict(test_data)
```

如果想动手实现 Adaboost 算法，并掌握如何使用 sklearn 来解决实际问题，可以尝试进入链接进行实战：https://www.educoder.net/shixuns/xkv4b6yc/challenges