# 5.3.2 用密度来聚类---DBSCAN

## 基本概念

在`DBSCAN`算法中，有两个基本的领域参数，分别为`eps`邻域和`Minpts`。

`eps`邻域表示的是在数据集`D`中与样本点$$x_i$$的距离不大于`eps`的样本。

样本点$$x_i$$的`eps`邻域如图所示：

![](6.jpg)

在图中，样本点$$x$$不在样本点$$x_i$$的`eps`邻域内。$$x_i$$的密度由$$x_i$$的`eps`邻域内的点的数量来估计。`Minpts`表示的是在样本点$$x_i$$的`eps`邻域内的最少样本点数目。基于邻域参数 **eps** 和 **Minpts**，在DBSCAN算法中将数据点分为一下三类：

1.**核心点**：若样本$$x_i$$的`eps`邻域内至少包含了 **Minpts** 个样本，则称样本点$$x_i$$为核心点。
2.**边界点**：若样本$$x_i$$的`eps`邻域内包含的样本数目小于 **Minpts**，但是它在其它核心点的邻域内，则称样本点$$x_i$$为边界点。
3.**噪音点**：指的是既不是核心点，又不是边界点的点。

![](7.jpg)

如上图，设置 **Minpts** 的值为`8`，对应的样本点$$x_1$$的`eps`邻域内包含了`9`个点，大于 **Minpts**，则样本点$$x_1$$为核心点。样本点$$x_2$$在样本点$$x_1$$的`eps`邻域内，且样本点$$x_2$$的`eps`邻域内只包含了`2`个样本点，小于 **Minpts**，则样本点$$x_2$$为边界点。样本点$$x_3$$为噪音点。

在`DBSCAN`算法中，还定义了如下的一些概念：

1.**直接密度可达**：若样本点$$x_j$$ 在核心点$$x_i$$的`eps`邻域内，则称样本点$$x_j$$从样本点$$x_i$$直接密度可达。
2.**密度可达**：若在样本点$$x_1$$和样本点$$x_n$$之间存在一序列

$$
x_2,..,x_{n-1}
$$

且$$x_{i+1}$$从$$x_i$$直接密度可达，则称$$x_n$$从$$x_1$$密度可达。
3.**密度相连**：对于样本点$$x_i$$和样本点$$x_j$$，若存在样本点$$x_k$$，使得$$x_i$$和$$x_j$$都从$$x_k$$密度可达，则称$$x_i$$和$$x_j$$密度相连。

![](8.jpg)

如上图，设置 **Minpts** 的值为`8`，则样本点$$x_1$$和$$x_2$$都是核心点，样本点$$x_3$$为边界点。样本点$$x_2$$在核心点$$x_1$$`eps`邻域内，则样本点$$x_2$$从样本点$$x_1$$直接密度可达。样本点$$x_3$$在在核心点$$x_2$$`eps`邻域内，则样本点$$x_3$$从样本点$$x_2$$直接密度可达。样本点$$x_1$$和$$x_3$$直接存在样本点$$x_2$$，且样本点$$x_2$$从样本点$$x_1$$直接密度可达，则样本点$$x_3$$从样本点$$x_1$$密度可达。

## DBSCAN算法原理

基于密度的聚类算法通过寻找被低密度区域分离的高密度区域，并将高密度区域作为一个簇。在`DBSCAN`算法中，聚类簇定义为：由密度可达关系导出的最大的密度相连样本集合。

## DBSCAN算法流程

在`DBSCAN`算法中，由核心对象出发，找到与该核心对象密度可达的所有样本形成一个聚类簇。`DBSCAN`算法流程如下：

`1`. 如果一个点的eps邻域包含多于MinPits个对象，则创建一个集合P作为核心对象的新簇。
`2`. 寻找核心对象的直接密度可达的对象，并合并为一个新的簇。
`3`. 直到没有点可以更新簇时算法结束。

## DBSCAN算法优点

根据`DBSCAN`算法原理及流程可以发现，`DBSCAN`算法在聚类时不需要自己设定簇的个数，而且能够发现任意形状的簇。还有一个优点就是，`DBSCAN`算法对噪音点不敏感，所以`DBSCAN`算法也常用来找寻异常数据。

## sklearn中的DBSCAN

`DBSCAN`的构造函数中有两个常用的参数可以设置：

- `eps`：`eps` 邻域半径大小
- `min_samples`：即 Minpts，`eps`邻域内样本最少数目


和`sklearn`中其他聚类器一样，`DBSCAN`不允许对新的数据进行预测，`DBSCAN`类中的`fit_predict`函数用于训练模型并获取聚类结果，`fit_predict`函数有一个向量输入：
- `X`：大小为 **[样本数量,特征数量]** 的`ndarray`，存放训练样本

`DBSCAN`的使用代码如下：

```python
from sklearn.cluster import DBSCAN
dbscan = DBSCAN(eps=0.5,min_samples =10)
result = dbscan.fit_predict(x)
```

如果想动手实现`DBSCAN`算法，并掌握如何使用`sklearn`来解决实际问题，可以尝试进入链接进行实战：https://www.educoder.net/shixuns/jfzo8xcu/challenges